<?php

/**
 * @file
 * Contains \Drupal\themr_components\Form\ComponentGenerator.
 */

namespace Drupal\themr_components\Form;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\ContentEntityType;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\themr_components\ComponentGenerator;
use Drupal\themr_components\PluginManager\ThemrFieldManager;
use Drupal\themr_components\Prompts;
use OpenAI\Client;
use OpenAI\Exceptions\ErrorException;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ComponentGenerationForm extends FormBase implements ContainerInjectionInterface {

  /**
   * Entity type manager.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The entity field manager.
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  /**
   * The entity type bundle info.
   */
  protected EntityTypeBundleInfoInterface $entityTypeBundleInfo;

  /**
   * The open ai client.
   */
  protected Client $openAi;

  /**
   * The component generator.
   */
  protected ComponentGenerator $componentGenerator;

  /**
   * The Themr Field manager.
   */
  protected ThemrFieldManager $themrFieldManager;

  /**
   * Constructor.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    EntityFieldManagerInterface $entityFieldManagerInterface,
    EntityTypeBundleInfoInterface $entityTypeBundleInfoInterface,
    Client $openAi,
    ComponentGenerator $componentGenerator,
    ThemrFieldManager $themrFieldManager,
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->entityFieldManager = $entityFieldManagerInterface;
    $this->entityTypeBundleInfo = $entityTypeBundleInfoInterface;
    $this->openAi = $openAi;
    $this->componentGenerator = $componentGenerator;
    $this->themrFieldManager = $themrFieldManager;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('openai.client'),
      $container->get('themr_components.generator'),
      $container->get('plugin.manager.themr_field_manager'),
    );
  }
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'themr_component_generator';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#attached']['library'][] = 'themr_components/excalidraw';
    $form['#attached']['library'][] = 'themr_components/excalidraw-loader';

    $form['component_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Component Name'),
      '#description' => $this->t('The name of the component you want to create.'),
      '#required' => TRUE,
    ];

    $form['generation_type'] = [
      '#title' => $this->t('Generation Type'),
      '#type' => 'select',
      '#options' => [
        '' => $this->t('Choose how to generate'),
        'library' => $this->t('Themr Component Library'),
        'private_library' => $this->t('Private Component Library'),
        'image' => $this->t('From Image'),
        'doodle_and_text' => $this->t('Doodle and Text'),
        'text' => $this->t('Describe with Text only'),
      ],
      '#default_value' => 'doodle_and_text',
      '#description' => $this->t('Choose the way you want to generate your components.'),
    ];

    $form['image'] = [
      '#type' => 'fieldset',
      '#states' => [
        'visible' => [
          ':input[name="generation_type"]' => ['value' => 'image'],
        ],
      ],
    ];

    $form['image']['image_upload'] = [
      '#type' => 'file',
      '#title' => $this->t('Component Image'),
      '#description' => $this->t('The image of the component you want to generate.'),
      '#states' => [
        'required' => [
          ':input[name="generation_type"]' => ['value' => 'image'],
        ],
      ],
    ];

    $form['image']['image_prompt'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Extra information'),
      '#description' => $this->t('If you want to give some extra information, please give it here.'),
    ];

    $form['doodle'] = [
      '#type' => 'fieldset',
      '#states' => [
        'visible' => [
          ':input[name="generation_type"]' => ['value' => 'doodle_and_text'],
        ],
      ],
    ];

    $form['doodle']['doodle'] = [
      '#markup' => '<h3>Doodle a component</h3><div id="doodle"></div>'
    ];

    $form['doodle']['doodle_base64'] = [
      '#type' => 'hidden',
      '#attributes' => [
        'id' => [
          'doodle-base-64',
        ],
      ],
    ];

    $form['doodle']['doodle_prompt'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Extra information'),
      '#description' => $this->t('If you want to give some extra information, please give it here.'),
    ];

    $form['list_view_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('This is a list view'),
      '#description' => $this->t('If this component should list and link to other entities, please check here and choose entity.'),
      '#states' => [
        'visible' => [
          ':input[name="generation_type"]' => ['!value' => ''],
        ],
      ],
    ];

    $entities = [];
    foreach ($this->getEntityTypesWithBundles() as $key => $data) {
      $entities[$key] = $data['name'];
    };

    $form['list_view_entity'] = [
      '#type' => 'select',
      '#title' => $this->t('List Content'),
      '#description' => $this->t('The content/entity types to list.'),
      '#options' => $entities,
      '#default_value' => 'node',
      '#states' => [
        'visible' => [
          ':input[name="list_view_enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['generate_css'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Generate CSS'),
      '#default_value' => TRUE,
      '#description' => $this->t('This will try to generate CSS as well. NOTE: This will take longer time and incure more costs in OpenAI. CSS will not be optimized, hire a frontender for that.'),
      '#states' => [
        'visible' => [
          ':input[name="generation_type"]' => ['!value' => ''],
        ],
      ],
    ];

    $form['actions']['fake-submit'] = [
      '#type' => 'button',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $formState) {
    $id = '';
    switch ($formState->getValue('generation_type')) {
      case 'image':
        $files = $this->getRequest()->files->get('files', []);
        $image = $files['image_upload'];
        $imageData = file_get_contents($image->getRealPath());
        $prompt = Prompts::imageComponentPrompt($formState->getValue('image_prompt'), 'image');
        $response = $this->openAiChatRequest($prompt, [], [
          'data:' . $image->getMimeType() . ';base64,' . base64_encode($imageData),
        ]);
        if (!empty($response['choices'][0]['message']['content'])) {
          $output = json_decode(str_replace(['```json', '```'], '', $response['choices'][0]['message']['content']), TRUE);
        }
        file_put_contents('/tmp/outputs.txt', $response['choices'][0]['message']['content'] . "\n\n", FILE_APPEND);
        //$output = json_decode(str_replace(['```json', '```'], '', $this->tempOutput()), TRUE);
        $id = $this->componentGenerator->generateComponent($formState->getValue('component_name'), $output);
        break;

      case 'doodle_and_text':
        $prompt = Prompts::doodleComponentGenerator($formState->getValue('doodle_prompt'), 'doodle');
        $response = $this->openAiChatRequest($prompt, [], [
          $formState->getValue('doodle_base64'),
        ]);
        if (!empty($response['choices'][0]['message']['content'])) {
          $output = json_decode(str_replace(['```json', '```'], '', $response['choices'][0]['message']['content']), TRUE);
        }
        file_put_contents('/tmp/outputs.txt', $response['choices'][0]['message']['content'] . "\n\n", FILE_APPEND);
        $id = $this->componentGenerator->generateComponent($formState->getValue('component_name'), $output);
        break;
    }

    if ($id) {
      $url = Url::fromRoute('themr_components.modify', ['component' => $id]);
      $formState->setRedirectUrl($url);
    }
  }

  /**
   * Temp.
   */
  public function tempOutput() {
    return '```json
[
  {
    "type": "div",
    "container-name": "Feature Component",
    "attributes": {
      "class": "row g-0 align-items-center themr-feature-component"
    },
    "css": {
      "background-color": "#F5F5F7"
    },
    "contains": [
      {
        "type": "div",
        "container-name": "Text Content Wrapper",
        "attributes": {
          "class": "col-md-6 p-5 themr-text-content-wrapper"
        },
        "contains": [
          {
            "type": "h2",
            "field-type": "string",
            "field-name": "Feature Title",
            "field-help-text": "Enter the title for the feature component.",
            "field-required": true,
            "field-example-value": "Turn data collection into an experience",
            "field-item-amount": 1,
            "field-form-weight": 0,
            "css": {
              "font-size": "36px",
              "font-weight": "300",
              "color": "#000000"
            },
            "attributes": {
              "class": "themr-feature-title themr-field"
            }
          },
          {
            "type": "p",
            "field-type": "text_long",
            "field-name": "Feature Description",
            "field-help-text": "Enter the description for the feature.",
            "field-required": false,
            "field-example-value": "Engage your audience with conversational forms & surveys — and get more data.",
            "field-item-amount": 1,
            "field-form-weight": 10,
            "css": {
              "font-size": "18px",
              "color": "#000000"
            },
            "attributes": {
              "class": "mb-4 themr-feature-description themr-field"
            }
          },
          {
            "type": "div",
            "container-name": "Button Group",
            "attributes": {
              "class": "themr-button-group"
            },
            "contains": [
              {
                "type": "a",
                "field-type": "link",
                "field-name": "Feature Primary Button",
                "field-help-text": "Set the primary action button link.",
                "field-required": false,
                "field-example-value": "Try it for free",
                "field-item-amount": 1,
                "field-form-weight": 20,
                "attributes": {
                  "class": "btn btn-dark me-3 themr-feature-primary-button themr-field",
                  "href": "#"
                }
              },
              {
                "type": "a",
                "field-type": "link",
                "field-name": "Feature Secondary Button",
                "field-help-text": "Set the secondary action button link.",
                "field-required": false,
                "field-example-value": "See examples",
                "field-item-amount": 1,
                "field-form-weight": 30,
                "attributes": {
                  "class": "btn btn-outline-dark themr-feature-secondary-button themr-field",
                  "href": "#"
                }
              }
            ]
          }
        ]
      },
      {
        "type": "div",
        "container-name": "Image Content Wrapper",
        "attributes": {
          "class": "col-md-6 themr-image-content-wrapper"
        },
        "contains": [
          {
            "type": "img",
            "field-type": "image",
            "field-name": "Feature Image",
            "field-help-text": "Upload an image for the feature component.",
            "field-required": true,
            "field-example-value": "",
            "field-item-amount": 1,
            "field-form-weight": 40,
            "max-image-width": 937,
            "max-image-height": 471,
            "attributes": {
              "class": "img-fluid themr-feature-image themr-field",
              "src": "",
              "alt": "Feature Image"
            }
          }
        ]
      }
    ]
  }
]
```';
  }

  /**
   * Get all entity types with bundle.
   *
   * @return array
   *   The entities and bundles.
   */
  public function getEntityTypesWithBundles() {
    $typesWithBundles = [];
    foreach($this->entityTypeManager->getDefinitions() as $entityName => $entityDefinition) {
      if ($entityDefinition instanceof ContentEntityType) {
        $typesWithBundles[$entityName] = [
          "machine_name" => $entityName,
          "name" => $entityDefinition->getLabel(),
          "bundles" => []
        ];
      }
    }
    foreach ($typesWithBundles as $entityTypekey => &$entityType) {
      foreach ($this->entityTypeBundleInfo->getBundleInfo($entityTypekey) as $bundle => $bundleInfo) {
        $entityType['bundles'][] = $bundle;
      }
    }
    return $typesWithBundles;
  }

  /**
   * Generate the OpenAI Chat request.
   *
   * @param string $prompt
   *   The prompt.
   * @param array $interpolatorConfig
   *   The interpolator config.
   * @param array $images
   *   Images to pass on to OpenAI Vision.
   *
   * @return string
   *   The response.
   */
  protected function openAiChatRequest($prompt, $config, $images = []) {
    $messages[] = [
      'role' => 'system',
      'content' => trim($prompt),
    ];
    $content[] = [
      "type" => "text",
      "text" => 'Generate the code for the component that looks exactly like this.'
    ];
    if (count($images)) {
      foreach ($images as $image) {
        $content[] = [
          "type" => "image_url",
          "image_url" => [
            "url" => $image,
          ],
        ];
      }
    }
    $messages[] = [
      'role' => 'user',
      'content' => $content,
    ];
    try {
      $response = $this->openAi->chat()->create([
        'model' => 'gpt-4o',
        'messages' => $messages,
        'max_tokens' => 4000,
      ]);
    }
    catch (ErrorException $e) {
      return $e->getMessage();
    }

    $result = $response->toArray();
    return $result;
  }
}
