<?php

/**
 * @file
 * Contains \Drupal\themr_components\Form\ComponentGenerator.
 */

namespace Drupal\themr_components\Form;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\themr_components\ComponentGenerator;
use Drupal\themr_components\PluginManager\ThemrFieldManager;
use OpenAI\Client;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ComponentModificationForm extends FormBase implements ContainerInjectionInterface {

  /**
   * Entity type manager.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The entity field manager.
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  /**
   * The entity type bundle info.
   */
  protected EntityTypeBundleInfoInterface $entityTypeBundleInfo;

  /**
   * The open ai client.
   */
  protected Client $openAi;

  /**
   * The component generator.
   */
  protected ComponentGenerator $componentGenerator;

  /**
   * The generators.
   */
  protected array $generators = [];

  /**
   * Constructor.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    EntityFieldManagerInterface $entityFieldManagerInterface,
    EntityTypeBundleInfoInterface $entityTypeBundleInfoInterface,
    Client $openAi,
    ComponentGenerator $componentGenerator,
    ThemrFieldManager $themrFieldManager,
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->entityFieldManager = $entityFieldManagerInterface;
    $this->entityTypeBundleInfo = $entityTypeBundleInfoInterface;
    $this->openAi = $openAi;
    $this->componentGenerator = $componentGenerator;
    foreach ($themrFieldManager->getDefinitions() as $definition) {
      $this->generators[$definition['id']] = $themrFieldManager->createInstance($definition['id']);
    }
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('openai.client'),
      $container->get('themr_components.generator'),
      $container->get('plugin.manager.themr_field_manager'),
    );
  }
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'themr_component_modification';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $component = NULL) {
    $form['#attached']['library'][] = 'themr_components/modify';
    $form['#attached']['library'][] = 'themr_components/aos-dev';

    $form['padding-top'] = [
      '#markup' => '<hr>',
    ];
    // Render the component.
    $form['wrapper'] = [
      '#type' => 'fieldset',
      '#attributes' => [
        'id' => 'component-wrapper',
      ],
    ];
    $form['wrapper']['component'] = $this->generateComponent($component);
    $form['padding-bottom'] = [
      '#markup' => '<hr>',
    ];

    // Load CSS.
    /*$cssFile = "/styles/components/$component";
    $form['css'] = [
      '#type' => 'inline_template',
      '#template' => '<link rel="stylesheet" media="all" href="{{ css_file }}" />',
      '#context' => [
        'css_file' => $cssFile,
      ],
    ];*/

    $form['component_name'] = [
      '#default_value' => $component,
      '#type' => 'value',
      '#value' => $component,
    ];

    $form['regenerate'] = [
      '#type' => 'button',
      '#value' => 'Regenerate',
      '#attributes' => [
        #'style' => 'display: none;'
      ],
      '#ajax' => [
        'callback' => [$this, 'regenerateComponent'],
        'event' => 'click',
        'wrapper' => 'component-wrapper',
        'prevent' => 'click',
      ],
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function regenerateComponent(array &$form, FormStateInterface $formState) {
    $form['wrapper']['component'] = $this->generateComponent($formState->getValue('component_name'));
    return $form['wrapper'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $formState) {

  }

  /**
   * Generate component with test data.
   */
  protected function generateComponent($componentType) {
    $storage = $this->entityTypeManager->getStorage('paragraph');
    $builder = $this->entityTypeManager->getViewBuilder('paragraph');
    $paragraph = $storage->create([
      'type' => $componentType,
    ]);
    $this->mockValues($componentType, $paragraph);

    return $builder->view($paragraph, 'full');
  }

  /**
   * Fill out mock values for one value.
   */
  protected function mockValues($componentType, &$entity) {
    $fields = $this->config('themr_components.component.' . $componentType)->get('fields');

    /** @var \Drupal\Core\Field\FieldDefinitionInterface[] */
    $entityFields = $this->entityFieldManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
    foreach ($entityFields as $fieldName => $fieldDefinition) {
      foreach ($fields as $fieldData) {
        if ($fieldData['field-name'] == $fieldDefinition->getLabel()) {
          $values = $this->generators[$fieldData['field-type']]->generateExampleValues($fieldDefinition->getConfig($entity->bundle()), $entity->bundle(), $fieldData['field-example-value'] ?? "");
          $entity->set($fieldDefinition->getName(), $values);
        }
      }
    }

  }

}
